import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import * as StatusStore from '@app/store/messenger/status';

import { of } from 'rxjs';
import {
  catchError,
  map,
  tap,
  switchMap,
  withLatestFrom
} from 'rxjs/operators';

import {
  dept,
  deptSuccess,
  deptFailure,
  myDeptUserSuccess,
  myDeptUser,
  myDeptUserFailure,
  deptUserSuccess,
  deptUser,
  selectedDept,
  deptUserFailure,
  selectedDeptSuccess,
  searchDeptUser,
  searchDeptUserSuccess,
  integrateSearchDeptUser,
  integrateSearchDeptUserSuccess,
  integrateSearchDeptUserFailure,
  searchDeptUserFailure
} from './actions';

import {
  QueryProtocolService,
  DeptInfo,
  SSVC_TYPE_QUERY_DEPT_DATA,
  SSVC_TYPE_QUERY_DEPT_RES,
  DeptData,
  UserInfoSS,
  SSVC_TYPE_QUERY_DEPT_USER_DATA,
  DeptUserData,
  SSVC_TYPE_QUERY_DEPT_USER_RES,
  DeptSearchType,
  DeptUserRequest
} from '@ucap-webmessenger/protocol-query';

import { Store, select } from '@ngrx/store';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { OrganizationService } from '@ucap-webmessenger/ui-organization';
import { NGXLogger } from 'ngx-logger';

@Injectable()
export class Effects {
  dept$ = createEffect(
    () => {
      let departmentInfoList: DeptInfo[];

      return this.actions$.pipe(
        ofType(dept),
        tap(() => {
          departmentInfoList = [];
        }),
        switchMap(req => {
          return this.queryProtocolService.dept(req).pipe(
            map(res => {
              switch (res.SSVC_TYPE) {
                case SSVC_TYPE_QUERY_DEPT_DATA:
                  departmentInfoList.push(...(res as DeptData).departmentInfos);
                  break;
                case SSVC_TYPE_QUERY_DEPT_RES:
                  departmentInfoList.sort((a, b) =>
                    a.order < b.order ? -1 : a.order > b.order ? 1 : 0
                  );
                  this.store.dispatch(
                    deptSuccess({
                      departmentInfoList
                    })
                  );
                  break;
              }
            }),
            catchError(error => of(deptFailure({ error })))
          );
        })
      );
    },
    { dispatch: false }
  );

  selectedDept$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(selectedDept),
        withLatestFrom(
          this.store.pipe(
            select(
              (state: any) =>
                state.account.authentication.loginRes as LoginResponse
            )
          )
        ),
        switchMap(([req, loginResInfo]) => {
          const isMyDept = req.seq === loginResInfo.departmentCode;
          return this.organizationService
            .getDeptUser({
              divCd: 'ORG' + (isMyDept ? '_MY' : ''),
              companyCode: loginResInfo.companyCode,
              seq: req.seq,
              search: '',
              searchRange: DeptSearchType.All,
              senderCompanyCode: loginResInfo.companyCode,
              senderEmployeeType: loginResInfo.userInfo.employeeType
            })
            .pipe(
              map(datas => {
                const userInfos: UserInfoSS[] = datas.userInfos;

                this.store.dispatch(
                  deptUserSuccess({
                    userInfos
                  })
                );

                if (!!isMyDept) {
                  this.store.dispatch(
                    myDeptUserSuccess({
                      userInfos
                    })
                  );
                } else {
                  // 검색 결과에 따른 프레즌스 조회.
                  const userSeqList: number[] = [];
                  userInfos.map(user => userSeqList.push(user.seq));
                  if (userSeqList.length > 0) {
                    this.store.dispatch(
                      StatusStore.bulkInfo({
                        divCd: 'orgtrSrch',
                        userSeqs: userSeqList
                      })
                    );
                  }
                }

                this.store.dispatch(selectedDeptSuccess({}));
              }),
              catchError(error => of(searchDeptUserFailure({ error })))
            );
        })
      );
    },
    { dispatch: false }
  );

  integrateSearchDeptUser$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(integrateSearchDeptUser),
        withLatestFrom(
          this.store.pipe(
            select(
              (state: any) =>
                state.account.authentication.loginRes as LoginResponse
            )
          )
        ),
        switchMap(([req, loginResInfo]) => {
          const pageListCount = 180;
          const pageCurrent = !!req.pageCurrent ? req.pageCurrent : 1;
          const request = {
            divCd: 'INT_SRCH',
            companyCode: req.companyCode,
            search: req.search,
            searchRange: DeptSearchType.All,
            senderCompanyCode: loginResInfo.companyCode,
            senderEmployeeType: loginResInfo.userInfo.employeeType,
            pageListCount,
            pageCurrent
          } as DeptUserRequest;

          let mergedUserList: UserInfoSS[] = !!req.userList ? req.userList : [];

          const userInfos: UserInfoSS[] = [];
          return this.queryProtocolService.deptUser(request).pipe(
            map(res => {
              switch (res.SSVC_TYPE) {
                case SSVC_TYPE_QUERY_DEPT_USER_DATA:
                  userInfos.push(...(res as DeptUserData).userInfos);
                  break;
                case SSVC_TYPE_QUERY_DEPT_USER_RES:
                  if (!!userInfos && userInfos.length > 0) {
                    mergedUserList = [...mergedUserList, ...userInfos];
                  }

                  // 검색 결과에 따른 프레즌스 조회.
                  const userSeqList: number[] = [];
                  userInfos.map(user => userSeqList.push(user.seq));
                  if (userSeqList.length > 0) {
                    this.store.dispatch(
                      StatusStore.bulkInfo({
                        divCd: 'inttrSrch',
                        userSeqs: userSeqList
                      })
                    );
                  }

                  // 재귀 할지 판단.
                  if (!!userInfos && userInfos.length >= pageListCount) {
                    // 추가 조회.
                    this.store.dispatch(
                      integrateSearchDeptUser({
                        ...req,
                        pageCurrent: pageCurrent + 1,
                        userList: mergedUserList
                      })
                    );
                  } else {
                    mergedUserList.sort((a, b) =>
                      a.order < b.order
                        ? -1
                        : a.order > b.order
                        ? 1
                        : a.name < b.name
                        ? -1
                        : a.name > b.name
                        ? 1
                        : 0
                    );

                    // 최종 조회건.
                    this.store.dispatch(
                      integrateSearchDeptUserSuccess({
                        userInfos: mergedUserList
                      })
                    );
                  }
                  break;
              }
            }),
            catchError(error => of(integrateSearchDeptUserFailure({ error })))
          );

          // return this.organizationService
          //   .getDeptUser(request)
          //   .pipe(
          //     map(datas => {
          //       const userInfos: UserInfoSS[] = datas.userInfos;
          //       this.store.dispatch(
          //         integrateSearchDeptUserSuccess({
          //           userInfos
          //         })
          //       );

          //       // 검색 결과에 따른 프레즌스 조회.
          //       const userSeqList: number[] = [];
          //       userInfos.map(user => userSeqList.push(user.seq));
          //       if (userSeqList.length > 0) {
          //         this.store.dispatch(
          //           StatusStore.bulkInfo({
          //             divCd: 'inttrSrch',
          //             userSeqs: userSeqList
          //           })
          //         );
          //       }
          //     }),
          //     catchError(error => of(integrateSearchDeptUserFailure({ error })))
          //   );
        })
      );
    },
    { dispatch: false }
  );

  searchDeptUser$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(searchDeptUser),
        withLatestFrom(
          this.store.pipe(
            select(
              (state: any) =>
                state.account.authentication.loginRes as LoginResponse
            )
          )
        ),
        switchMap(([req, loginResInfo]) => {
          const pageListCount = 180;
          const pageCurrent = !!req.pageCurrent ? req.pageCurrent : 1;
          const request = {
            divCd: 'ORG_SRCH',
            companyCode: req.companyCode,
            search: req.search,
            searchRange: DeptSearchType.All,
            senderCompanyCode: loginResInfo.companyCode,
            senderEmployeeType: loginResInfo.userInfo.employeeType,
            pageListCount,
            pageCurrent
          } as DeptUserRequest;

          let mergedUserList: UserInfoSS[] = !!req.userList ? req.userList : [];

          const userInfos: UserInfoSS[] = [];
          return this.queryProtocolService.deptUser(request).pipe(
            map(res => {
              switch (res.SSVC_TYPE) {
                case SSVC_TYPE_QUERY_DEPT_USER_DATA:
                  userInfos.push(...(res as DeptUserData).userInfos);
                  break;
                case SSVC_TYPE_QUERY_DEPT_USER_RES:
                  if (!!userInfos && userInfos.length > 0) {
                    mergedUserList = [...mergedUserList, ...userInfos];
                  }

                  // 검색 결과에 따른 프레즌스 조회.
                  const userSeqList: number[] = [];
                  userInfos.map(user => userSeqList.push(user.seq));
                  if (userSeqList.length > 0) {
                    this.store.dispatch(
                      StatusStore.bulkInfo({
                        divCd: 'orgtrSrch',
                        userSeqs: userSeqList
                      })
                    );
                  }

                  // 재귀 할지 판단.
                  if (!!userInfos && userInfos.length >= pageListCount) {
                    // 추가 조회.
                    this.store.dispatch(
                      searchDeptUser({
                        ...req,
                        pageCurrent: pageCurrent + 1,
                        userList: mergedUserList
                      })
                    );
                  } else {
                    mergedUserList.sort((a, b) =>
                      a.order < b.order
                        ? -1
                        : a.order > b.order
                        ? 1
                        : a.name < b.name
                        ? -1
                        : a.name > b.name
                        ? 1
                        : 0
                    );

                    // 최종 조회건.
                    this.store.dispatch(
                      searchDeptUserSuccess({
                        userInfos: mergedUserList
                      })
                    );
                  }
                  break;
              }
            }),
            catchError(error => of(deptUserFailure({ error })))
          );
        })
      );
    },
    { dispatch: false }
  );

  deptUser$ = createEffect(
    () => {
      let userInfos: UserInfoSS[];

      return this.actions$.pipe(
        ofType(deptUser),
        tap(() => {
          userInfos = [];
        }),
        switchMap(req => {
          return this.queryProtocolService.deptUser(req).pipe(
            map(res => {
              switch (res.SSVC_TYPE) {
                case SSVC_TYPE_QUERY_DEPT_USER_DATA:
                  userInfos.push(...(res as DeptUserData).userInfos);
                  break;
                case SSVC_TYPE_QUERY_DEPT_USER_RES:
                  userInfos.sort((a, b) =>
                    a.order < b.order
                      ? -1
                      : a.order > b.order
                      ? 1
                      : a.name < b.name
                      ? -1
                      : a.name > b.name
                      ? 1
                      : 0
                  );

                  this.store.dispatch(
                    deptUserSuccess({
                      userInfos
                    })
                  );
                  this.store.dispatch(
                    myDeptUserSuccess({
                      userInfos
                    })
                  );
                  break;
              }
            }),
            catchError(error => of(deptUserFailure({ error })))
          );
        })
      );
    },
    { dispatch: false }
  );

  myDeptUser$ = createEffect(
    () => {
      let userInfos: UserInfoSS[];

      return this.actions$.pipe(
        ofType(myDeptUser),
        tap(() => {
          userInfos = [];
        }),
        switchMap(req => {
          return this.queryProtocolService.deptUser(req).pipe(
            map(res => {
              switch (res.SSVC_TYPE) {
                case SSVC_TYPE_QUERY_DEPT_USER_DATA:
                  userInfos.push(...(res as DeptUserData).userInfos);
                  break;
                case SSVC_TYPE_QUERY_DEPT_USER_RES:
                  userInfos.sort((a, b) =>
                    a.order < b.order
                      ? -1
                      : a.order > b.order
                      ? 1
                      : a.name < b.name
                      ? -1
                      : a.name > b.name
                      ? 1
                      : 0
                  );

                  this.store.dispatch(
                    deptUserSuccess({
                      userInfos
                    })
                  );
                  this.store.dispatch(
                    myDeptUserSuccess({
                      userInfos
                    })
                  );
                  break;
              }
            }),
            catchError(error => of(myDeptUserFailure({ error })))
          );
        })
      );
    },
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService,
    private organizationService: OrganizationService,
    private sessionStorageService: SessionStorageService,
    private logger: NGXLogger
  ) {}
}
